@@ -22,6 +22,8 @@ import weakref
 import exputils
 from stimulus import Stimulus
 
+import pygame
+
 # Get the PyEPL directory (used below to find the default font)
 pyepldir = os.path.abspath(os.path.dirname(__file__))
 
@@ -191,7 +193,6 @@ class Image(UniquelyConstructed, Carvable, Showable, Stimulus):
         """
         if self.img:
             return self.img_unscaled
-
         self.load()
         r = self.img_unscaled
         self.unload()
@@ -480,6 +481,294 @@ class Image(UniquelyConstructed, Carvable, Showable, Stimulus):
             return timestamp
 
 
+# New Movie Class
+class Movie(UniquelyConstructed, Showable, Stimulus):
+    """
+    Represents a movie.  In PyEPL, movies are rendered onto an image
+    surface, which you must first show (via one of the many show
+    methods in the VideoTrack).
+
+    Once a movie surface is shown, you can start and stop playback
+    with the playMovie and stopMovie methods of the VideoTrack.
+
+    When you are done playing a movie, you can then unshow the image
+    where the movie was projected.
+    """
+    def __uinit__(self, x):
+        """
+        Creates a new Movie object.  It uses pygame to play the movie,
+        so it only supports mpg files.
+
+        INPUT ARGS:
+          x- Data used to construct Movie.
+             Can be the filename of an movie file,
+             or a pygame.movie.Movie object.
+        ...
+        """
+        Showable.__init__(self)
+
+        if isinstance(x, pygame.movie.MovieType):
+            # if the parameter is a Movie object, just wrap it
+            self.movie = x
+            self.filename = None
+            self.img = None
+        else:
+            # otherwise, assume it's a filename
+            self.filename = x
+            self.movie = None
+            self.img = None
+        
+        # set empty logline
+        self.logLineStr = 'MOVIE'
+
+    def load(self):
+        """
+        After calling this method, the image data is gauranteed to be
+        loaded into primary memory.
+
+        If keepLoadedAfterSmartShow is set to True, the image will not
+        be unloaded after it is shown with SmartShow.
+        """
+        if self.movie is None:
+            # create the movie instance
+            self.movie = pygame.movie.Movie(self.filename)
+
+        if self.img is None:
+            # Create a surface where pygame will render the movie
+            self._render_surf = pygame.Surface(self.movie.get_size())
+
+            # Tell the movie to render frames to that new surface
+            self.movie.set_display(self._render_surf)
+
+            # Create a LowImage for performing the OpenGL blitting of the
+            # movie frames to the actual OpenGL surface that we use
+            self.img = hardware.graphics.LowImage(self._render_surf)
+    def unload(self):
+        """
+        After calling this method, the Movie's data is unloaded from memory.
+        """
+        # stop playing if we are
+        self.stop()
+        
+        # unload by removing our internal reference to the LowImage object
+        self._render_surf = None
+        self.img = None
+        self.movie = None
+        
+    def isLoaded(self):
+        """
+        Checks if this object is actually loaded into memory.
+
+        OUTPUT ARGS:
+          b- True if the object is loaded in memory, False if not.
+        """
+        # check whether or not it's loaded by seeing if we have an
+        # internal reference to a LowImage object
+        return not self.img is None
+    
+    def __getstate__(self):
+        """
+        Don't give the image data to pickle.
+        """
+        # remove the Movie and LowImage objects before pickle looks at
+        # the data (because LowImages are not picklable)
+        d = self.__dict__.copy()
+        for todel in ['img','movie','_render_surf']:
+            try:
+                del d[todel]
+            except KeyError:
+                pass
+        return d
+    def __setstate__(self, state):
+        """
+        After unpickling, the movie is NOT loaded.
+        """
+        # unpickle with None as self.image (since we left it out when
+        # we pickled)
+        self.__dict__.update(state)
+        self.img = None
+        self.movie = None
+        self._render_surf = None
+
+    def export(self, archive, namehint):
+        """
+        Export image.
+        """
+        raise NotImplementedError #...
+    def setLogLine(self):
+        """
+        Set the ID for the video log.
+
+        XXX IMPORTANT XXX
+        We will eventually have to call this every time the movie is changed.
+        XXX IMPORTANT XXX
+        """
+        # get the movie's pixel size
+        xs, ys = self.getSize()
+
+        # return the log line
+        self.logLineStr = "MOVIE\t%s\t%d\t%d\t%d" % \
+                          (self.filename, xs, ys, self.getTotalTime())
+    def logLine(self):
+        """
+        Identify this movie for the video log.
+        """
+        return self.logLineStr
+    def getSize(self):
+        """
+        Return an x, y tuple for movie dimensions.
+        """
+        # make sure the image is loaded
+        self.load()
+        
+        # return the size of the LowImage
+        return self.img.getSize()
+    def show(self, x, y):
+        """
+        Put this image on the screen at x, y.
+        """
+        # make sure the image is loaded
+        self.load()
+
+        # see if we have to update the frame
+        if self.hasNewFrame():
+            self._updateFrame()
+        
+        # have the LowImage draw itself at the specified coordinates
+        self.img.show(x, y)
+
+        # set the logLine
+        self.setLogLine()
+
+    def hasNewFrame(self):
+        """
+        See if the movie has a new frame ready.
+        """
+        if not self.isLoaded():
+            self.load()
+            
+        return self.movie.get_frame() != self._prevFrame
+            
+    def _updateFrame(self):
+        """
+        Update the surface with the current movie frame.
+        """
+        # set the gl_texture to be dirty so it can be updated
+        self.img.gl_texture_dirty = True
+
+        # clean the image to which the movie will render
+        # this will render the new frame to the surface
+        self.img.cleanGLTexture()
+
+        # save the current frame
+        self._prevFrame = self.movie.get_frame()
+        
+
+    def play(self,loops=0):
+        
+        # make sure we are loaded
+        self.load()
+
+        # reset the previous frame
+        self._prevFrame = None
+        self._latest_timestamp = None
+        
+        # start playing
+        self.movie.play(loops)
+                
+    def pause(self):
+        # only pause if was running
+        if self.movie and self.movie.get_busy():
+            # pause it
+            self.movie.pause()
+#             # remove the playing callback
+#             removePollCallback(self._playMovieCallback)
+
+    def stop(self):
+        # stop the movie if playing
+        if self.movie and self.movie.get_busy():
+            self.movie.stop()
+
+#         # remove the poll callback
+#         removePollCallback(self._playMovieCallback)
+
+        # reset the previous frame
+        self._prevFrame = None
+        
+    def getCurrentFrame(self):
+        if self.movie:
+            return self.movie.get_frame()
+        
+    def getElapsedTime(self):
+        if self.movie:
+            return long(self.movie.get_time()*1000)
+        
+    def getTotalTime(self):
+        if self.movie:
+            return long(self.movie.get_length()*1000)
+
+    def getRemainingTime(self):
+        if self.movie:
+            return long((self.movie.get_length() -
+                         self.movie.get_time())*1000)
+
+    def present(self, clk = None, duration = None, jitter = None, bc = None, minDuration = None):
+        """
+        Present an movie on the screen.  If a ButtonChooser is
+        provided, present ignores the jitter and clears the stimulus
+        once a button is pressed or the duration waiting for a
+        response is exceeded.
+
+        INPUT ARGS:
+          clk- Optional PresentationClock to update.
+          duration/jitter- Duration to display the stimulus.
+          bc- Optional ButtonChooser object.
+          minDuration- Used in combination with bc to specify a minimum
+                       time that a stimulus must be presented before a
+                       keypress to the bc is accepted.
+          
+        OUTPUT ARGS:
+          timestamp- time and latency of when the movie came on the screen.
+          button- Button pressed if we passed in bc.
+          bc_time- Time and latency of when the button was pressed (if provided)
+        """
+
+        v = VideoTrack.lastInstance()
+        
+        # get the clock if needed
+        if clk is None:
+            clk = exputils.PresentationClock()
+
+        # do the auto delay if no duration is provided
+        if not bc and not duration:
+            doDelay = True
+        else:
+            doDelay = False
+        # show the image
+        shown_movie = v.showCentered(self)
+        # show the movie
+        timestamp = v.playMovie(self,t=clk,doDelay=doDelay)
+
+        if bc:
+            # wait for button press
+            button,bc_time = bc.waitWithTime(minDuration,duration,clk)
+        elif duration:
+            clk.delay(duration,jitter)
+
+        # stop the movie
+        v.stopMovie(self,t=clk)
+        
+        # unshow that image
+        v.unshow(shown_movie)
+        v.updateScreen(clk)
+
+        if bc:
+            return timestamp,button,bc_time
+        else:
+            return timestamp
+
+
+
 class Font(MediaFile):
     """
     Represents a graphical font.
@@ -1068,6 +1357,10 @@ class VideoTrack(textlog.LogTrack):
 
         # list of calls to make after the screen is updated
         self.do_after_update = []
+
+        # list of currently playing movies
+        self._playing_movies = []
+
     def startLogging(self):
         """
         Begin logging all basic video output.
@@ -1086,12 +1379,15 @@ class VideoTrack(textlog.LogTrack):
 
         # save the resolution
         self.screensize = hardware.getResolution()
+        
+        self.showFPS = hardware.getShowFPS()
     def stopService(self):
         """
         Finalize video output.
         """
         # close the window
         hardware.stopVideo()
+        
     def logResolution(self):
         """
         Note the resolution and windowedness in the video log.
@@ -1137,16 +1433,24 @@ class VideoTrack(textlog.LogTrack):
         """
         # return the saved resolution
         return self.screensize
-    def clear(self, color = None):
+    def clear(self, color = None, keep_background=True):
         """
         Clear basic video.  Make a color background or no background
         in unspecified.
         """
-        # clear the pending list
-        self.pending = []
-
-        # since the list is empty now, there are no active showables in it
-        self.activepending = 0
+        # clear by unshowing all the pending items
+        # get list of pending items
+        if color != None or keep_background==False:
+            # remove them all
+            toclear = [p[0] for p in self.pending]
+        else:
+            # keep any background color they have set
+            toclear = []
+            for p in self.pending:
+                if not isinstance(p[1], SolidBackground):
+                    toclear.append(p[0])
+        # unshow what we've selected
+        self.unshow(*toclear)
 
         # if a color is provided we want to show that after we've removed everything else...
         if color != None:
@@ -1174,17 +1478,36 @@ class VideoTrack(textlog.LogTrack):
         
         # return the shown
         return shown
-    def showProportional(self, showable, x, y):
+    def showProportional(self, showable, x, y, constrain=True):
         """
-        Like show, but position is proportional (i.e. 0.5, 0.5 is
-        centered).
+        showProportional displays a showable, with its position
+        specified as a proportion of the width and height of the whole
+        screen, such that (0.5,0.5) is the center.
+
+        The original (default) implementation of this was designed so
+        that objects would be placed at the edge (but not spilling
+        out) of the screen when the proportion was set to 0 or
+        1. Leave constrain=True to keep this behavior.
+
+        However, in retrospect, it makes more sense for these
+        proportion values to refer to the center of the object, such
+        that the object is 50% off the screen when the proportions are
+        0 or 1. For this behavior, set constrain=False. See
+        http://sourceforge.net/forum/forum.php?thread_id=1871462&forum_id=548620
+        for a clear demonstration of these two behaviors.
+        
         """
         # get the size of the showable
         objectsize = showable.getSize()
 
-        # calculation the correct pixel position (for the upper left corner) based on the showable size and the screen size
-        position = (int((self.screensize[0] - objectsize[0]) * x), int((self.screensize[1] - objectsize[1]) * y))
-
+        if constrain:
+            # calculate the correct pixel position (for the upper left corner) based on the showable size and the screen size
+            position = (int((self.screensize[0] - objectsize[0]) * x), int((self.screensize[1] - objectsize[1]) * y))
+        else:
+            # center the object at the specified position without the
+            # constraint that the object must be fully in the screen
+            position = (int((self.screensize[0] * x) - (objectsize[0] / 2)), int((self.screensize[1] * y) - (objectsize[1] / 2))) 
+            
         # show the showable at the calculated position, returning the Shown object
         return self.show(showable, *position)
     def showCentered(self, showable):
@@ -1391,7 +1714,8 @@ class VideoTrack(textlog.LogTrack):
 
         # update the screen at the calculated time
 	# now passes in self as a video track instance
-        r = hardware.makeVideoChanges(t,self)  
+        #r = hardware.makeVideoChanges(t,self)  
+        r = hardware.makeVideoChanges(t)  
         
         for fnc, targs, dargs in self.do_after_update:
             # perform all waiting do_after_update calls
@@ -1420,16 +1744,22 @@ class VideoTrack(textlog.LogTrack):
 
         # return the timestamp
         return r
-    def renderLoop(self, f):
+    def renderLoop(self, f, *pargs, **kwargs):
         """
         There has to be a render loop when active drawables are
         displayed.
         """
         # mark the start of the render loop in the log
         self.logMessage("ENTERLOOP")
+        
+        prev_last_updated = self.last_updated
+        fpsFrameCount = 1
+        fpsMinimumFrames = 50
+        fpsText = None
+        t_delta = 0.0
 
         # keep calling f until it returns a false value...
-        while f(self.last_updated):
+        while f(self.last_updated, *pargs, **kwargs):
             # after each call to f:
             # poll for events
             hardware.pollEvents()
@@ -1448,9 +1778,26 @@ class VideoTrack(textlog.LogTrack):
             for shown, showable, x, y in self.onscreen:
                 # ...showing everything in order
                 showable.show(x, y)
+            
+            # Show FPS
+            if self.showFPS:
+                t1 = prev_last_updated[0]
+                t2 = self.last_updated[0]
+                t_delta += t2 - t1
+                if t_delta and fpsFrameCount >= fpsMinimumFrames:
+                    fps = fpsFrameCount * 1000.0 / t_delta
+                    fpsText = Text("%.1fFPS"  % fps)
+                    fpsFrameCount = 1
+                    t_delta = 0.0
+                else:
+                    fpsFrameCount += 1
+                if fpsText:
+                    fpsText.show(0, 0)
 
             # update the screen at the calculated time
-            self.last_updated = hardware.makeVideoChanges(t,self)
+            prev_last_updated = self.last_updated
+            #self.last_updated = hardware.makeVideoChanges(t, self)
+            self.last_updated = hardware.makeVideoChanges(t)
 
             # log the contents of the display
             self.logDisplay(self.last_updated)
@@ -1633,3 +1980,158 @@ class VideoTrack(textlog.LogTrack):
 	red, green, blue = zip(*gvals)
 
 	return red,green,blue
+
+    def playMovie(self,movie,t=None,doDelay=True,loops=0):
+        """
+        Start playback of a movie object.
+
+        INPUT ARGS:
+           movie- Instance of a Movie object.
+           t- Time or presentation clock instance to control onset time.
+           doDelay- Whether to advance the clock the length of the movie.
+                    Defaults to True
+           loops- Number of times the movie will be repeated. A value of
+                  -1 will repeat forever.
+
+        OUTPUT ARGS:
+           timestamp- When the first frame played
+        """
+        # make sure it's not in the playing movies
+        if movie in self._playing_movies:
+            raise RuntimeError('You can only play a movie instance once.')
+        
+        # handle timing
+	clk = None
+        if t is None:
+            # if t is None, use the current time
+            t = timing.now()
+        elif isinstance(t, exputils.PresentationClock):
+            # if t is a PresentationClock, use the time from that clock
+            clk = t
+            t = t.get()
+
+        # are there any movies already playing
+        numPlaying = len(self._playing_movies)
+        
+        (timeInterval, start_timestamp) = timing.timedCall(t,
+                                                           self._startMovie,
+                                                           movie,loops)
+        
+        # start the callback if it's not already started
+        if numPlaying == 0:
+            hardware.addPollCallback(self._playingMovieCallback)
+
+        # log it
+        self.logMessage("%s\t%s\t%s" % \
+                        ("MOVIE_PLAY",movie.filename,
+                         movie.getRemainingTime()), timeInterval)
+
+        # tare the clock to the start time if we have a clock
+        if doDelay and clk:
+            # accumulate the error
+            clk.accumulatedTimingError += start_timestamp[0]-t
+            # tare the clock and delay the proper amount.
+            # I'm basing this on when the movie.play began, not when
+            # the first frame was rendered.
+            clk.tare(timeInterval[0])
+            clk.delay(movie.getRemainingTime())
+
+        # return the start time
+        return start_timestamp
+
+    def _startMovie(self,movie,loops=0):
+        # start playing the movie
+        movie.play(loops)
+
+        # add it to the playing movies
+        self._playing_movies.append(movie)
+        
+        # call the callback once to get the starting time
+        self._playingMovieCallback()
+        return self.last_frame_timestamp
+
+    def _playingMovieCallback(self):
+        # loop over playing movies to see if we have a new frame
+        hasNewFrame = False
+        for mov in self._playing_movies:
+            if mov.movie.get_busy():
+                if mov.hasNewFrame():
+                    hasNewFrame = True
+            else:
+                # we are done with this movie, so remove it from the
+                # list and tell it to stop
+                self.stopMovie(mov)
+
+        # see if we need to refresh the screen
+        if hasNewFrame:
+            # see if restart logging
+            if self.logall:
+                self.logall = False
+                restartLogging = True
+            else:
+                restartLogging = False
+            self.last_frame_timestamp = self.updateScreen()
+            if restartLogging:
+                self.logall = True
+
+    def stopMovie(self,movie,t=None):
+        # make sure it was playing
+        if movie in self._playing_movies:
+            # stop the movie at the specified time
+            (timeInterval,res) = timing.timedCall(t,
+                                              movie.stop)
+
+            # remove it from the playlist
+            # not sure why it's sometimes not there to remove
+            try:
+                self._playing_movies.remove(movie)
+            except:
+                print self._playing_movies
+                pass
+            
+            # see if remove callback
+            if len(self._playing_movies) == 0:
+                # none left to play, so stop callback
+                hardware.removePollCallback(self._playingMovieCallback)
+
+            self.logMessage("%s\t%s\t%s" % \
+                            ("MOVIE_STOP",movie.filename,
+                             movie.getRemainingTime()), timeInterval)
+
+    def stopAllMovies(self):
+        for movie in self._playing_movies:
+            # stop the movie
+            (timeInterval,res) = timing.timedCall(t,
+                                              movie.stop)
+
+            # remove it from the playlist
+            self._playing_movies.remove(movie)
+
+            # see if remove callback
+            if len(self._playing_movies) == 0:
+                # none left to play, so stop callback
+                hardware.removePollCallback(self._playingMovieCallback)
+
+            self.logMessage("%s\t%s\t%s" % \
+                            ("MOVIE_STOP",movie.filename,
+                             movie.getRemainingTime()), timeInterval)
+
+        
+    def pauseMovie(self,movie,t=None):
+        # make sure it was playing
+        if movie in self._playing_movies:
+            # pause the movie
+            (timeInterval,res) = timing.timedCall(t,
+                                              movie.pause)
+
+            # remove it from the playlist
+            self._playing_movies.remove(movie)
+
+            # see if remove callback
+            if len(self._playing_movies) == 0:
+                # none left to play, so stop callback
+                hardware.removePollCallback(self._playingMovieCallback)
+
+            self.logMessage("%s\t%s\t%s" % \
+                            ("MOVIE_PAUSE",movie.filename,
+                             movie.getRemainingTime()), timeInterval)
