@@ -8,9 +8,14 @@
 #include <string>
 #include <iostream>
 
-using std::string;
-using std::cerr;
-using std::endl;
+#include <cstdlib>
+#include <cstring>
+#include <math.h>
+
+using namespace std;
+/* using std::string; */
+/* using std::cerr; */
+/* using std::endl; */
 
 class Parallel {
   int fd;
