@@ -15,17 +15,16 @@
 #ifndef __EPLSOUND_H
 #define __EPLSOUND_H
 
-#include <iostream>
 #include "RtAudio.h"
 #include "MyType.h"
 #include "fifo.h"
 
-using namespace std;
-
 class audioBuffer
 {
  public:
-  audioBuffer(long recLen, long playLen, int inRecChans, int inPlayChans, int samprate);
+  audioBuffer(long recLen, long playLen, 
+	      unsigned int inRecChans, unsigned int inPlayChans, 
+	      unsigned int samprate);
   ~audioBuffer();
   fifo *recBuf;
   fifo *playBuf;
@@ -37,13 +36,15 @@ class audioBuffer
   long samplesPlayed;
 };
 
+// This should probably match what's in MyType.h
 #define FORMAT RTAUDIO_SINT16
 
 class eplSound 
 {
  public:
-  eplSound(long recLen=REC_BUF_LEN, long playLen=PLAY_BUF_LEN, int sampleRate=SAMPLE_RATE, 
-	   int bufSize=BUF_SIZE);
+  eplSound(long recLen=REC_BUF_LEN, long playLen=PLAY_BUF_LEN, 
+	   unsigned int sampleRate=SAMPLE_RATE, 
+	   unsigned int bufSize=BUF_SIZE);
   ~eplSound();
   long append(MY_TYPE *newdata, long length, int overwrite, float ampFactor);
   long consume(MY_TYPE *newdata, long length);
@@ -58,35 +59,38 @@ class eplSound
   int getRecChans();
   int getPlayChans();
   int getSampleRate();
+  long getBufferUsed();
   static const int SCALE=32767;  
-  static const int SAMPLE_RATE=44100;
+  static const unsigned int SAMPLE_RATE=44100;
   static const int SAMPLE_SILENCE=0;
-  static const int BUF_SIZE=256;
+  static const unsigned int BUF_SIZE=256;
   static const int NUM_INTERNAL_BUFF=4;
   static const int DEVICE=0;
   static const int FORMAT_SIZE=2;
-  static const int PLAY_BUF_LEN=60;
-  static const int REC_BUF_LEN=60;
+  static const long PLAY_BUF_LEN=60;
+  static const long REC_BUF_LEN=60;
   static const int NUM_CHANNELS=2;
  private:
-  RtAudio *audio;
-  RtAudio *playaudio;
-  RtAudio *recaudio;
-  int isDuplex;
-  int playChans;
-  int recChans;
-  int rate;
-  int bufferSize;
-  int playDevice;
-  int recDevice;
-  int sampleRate;
-  int streaming;
+  RtAudio audio;
+  //RtAudio *playaudio;
+  //RtAudio *recaudio;
+  //int isDuplex;
+  unsigned int playChans;
+  unsigned int recChans;
+  //int rate;
+  unsigned int bufferSize;
+  unsigned int playDevice;
+  unsigned int recDevice;
+  unsigned int sampleRate;
+  unsigned int streaming;
   audioBuffer *data;
 };
 
 // audio callback function
-int inout(char *buffer, int buffer_size, void *data);
-int playcall(char *buffer, int buffer_size, void *data);
-int reccall(char *buffer, int buffer_size, void *data);
+int inout( void *outputBuffer, void *inputBuffer, unsigned int nBufferFrames,
+           double streamTime, RtAudioStreamStatus status, void *data );
+//int inout(char *buffer, int buffer_size, void *data);
+//int playcall(char *buffer, int buffer_size, void *data);
+//int reccall(char *buffer, int buffer_size, void *data);
 
 #endif
