@@ -144,9 +144,36 @@ class AudioClip(Stimulus):
         else:
             # keep the clock advanced the duration of the sound
             return timestamp
+    
+    def __add__(self,clip):
+        """
+        Add two AudioClips together, aligning to the beginning of both
+        and returning a clip the length of the longer clip.
+        """
+
+        # convert from str to numpy array
+        base = self.stripChannel(self.snd)
+        toadd = self.stripChannel(clip.snd)
+        base = numpy.array(struct.unpack(str(len(base)/self.sampleWidth) + 'h', base), dtype=numpy.int16)
+	toadd = numpy.array(struct.unpack(str(len(toadd)/clip.sampleWidth) + 'h', toadd), dtype=numpy.int16)
+
+        # see the diff in length
+        clipdiff = len(base) - len(toadd)
+
+        if clipdiff > 0:
+            # append zeros to the clip
+            toadd = numpy.concatenate([toadd,numpy.zeros(clipdiff)])
+        elif clipdiff < 0:
+            # append zeros to the base
+            base = numpy.concatenate([base,numpy.zeros(-clipdiff)])
+
+        newclip = base+toadd*.5
+        newclip = self.duplicateChannel(newclip)
+        return AudioClip(newclip)
+
 
 class Beep(AudioClip):
-    def __init__(self, freq, duration, risefalltime = 0):
+    def __init__(self, freq, duration, risefalltime = 0, scalePercent = 0.8):
 	"""
 	Generate a beep of desired frequency, duration, and rise/fall
 	time.  Format of beep is in 16bit int samples.
@@ -157,11 +184,14 @@ class Beep(AudioClip):
           risefalltime- length of time (in ms.) for beep to rise from
             silence to full volume at beginning, and fall to no volume
             at end.
+          scalePercent- Percent of the max audio range for the beep (defaults to .8).
 
 	  """
 	AudioClip.__init__(self)
 	a = AudioTrack.lastInstance()
-	scale = a.eplsound.SCALE
+        
+        # set the scale
+	scale = a.eplsound.SCALE * scalePercent
 
 	# Do some rate and ms conversions
 	sampCycle = int(self.RESAMPLEDRATE/freq)
@@ -583,6 +613,7 @@ class AudioTrack(textlog.LogTrack):
 
         else:
             dur = 0
+            timeInterval = (t,0)
             
         # log message        
         self.logMessage("%s\t%s\t%s" % ("P",shortName,dur), timeInterval)
@@ -621,18 +652,159 @@ class AudioTrack(textlog.LogTrack):
 		if (self.eplsound.getSamplesPlayed()*self.eplsound.NUM_CHANNELS)>=self.total_samples:
 		    self.playStop()
 
-    def playStop(self, doUnload=True):
+    def playLoopStop(self, doUnload=True):
 	if self.playing:
 	    self.playing = False
-	    removePollCallback(self.__playCallback__)           
+	    removePollCallback(self.__playLoopCallback__)           
+        
+        # clear the sound buffer to stop playing
+	self.eplsound.clear()
 
 	if isinstance(self.currentClip, FileAudioClip) and self.currentClip.isLoaded() and doUnload:
 	    self.currentClip.unload()
 	    
+	return self.eplsound.getSamplesPlayed()
+
+
+    def playStop(self, doUnload=True):
+	if self.playing:
+	    self.playing = False
+	    removePollCallback(self.__playCallback__)           
+        
+        # clear the sound buffer to stop playing
 	self.eplsound.clear()
 
+	if isinstance(self.currentClip, FileAudioClip) and self.currentClip.isLoaded() and doUnload:
+	    self.currentClip.unload()
+	    
+
 	return self.eplsound.getSamplesPlayed()
 
+
+    def playLoop(self, soundClip, t = None, ampFactor=1.0, doDelay=True):
+        """
+        Play an AudioClip and return the time and latency of when the
+        sound played.
+
+        INPUT ARGS:
+          soundClip- AudioClip object of the sound to be played
+          t- Optional PresentationClock for timing.
+          ampFactor- Optional amplification of sound.  (default value is 1)
+          doDelay- Optionally do not tare and move the presentation clock
+            forward.  Defaults to True (moving the clock forward)
+          
+        OUTPUT ARGS:
+          timestamp- time and latency when sound playing began.
+          
+        """          
+
+	# Must be sure to not get multiple callbacks at once, so
+	# playing a soundclip while another is running causes that 
+	# other one to stop immediately, even if it is not done playing.
+	# self.playStop()
+
+	# handle special case: if it's a FileAudioClip and needs loading,
+	# load it.
+	self.currentClip = soundClip
+        if isinstance(soundClip, FileAudioClip):
+	    if not soundClip.isLoaded():
+		# load and append the sound
+		soundClip.load()
+	    # for logging
+	    shortName = soundClip.filename
+	else:
+	    shortName = "NOFILE"
+
+        if isinstance(t, exputils.PresentationClock):
+            clk = t
+	else:
+	    clk = exputils.PresentationClock()
+	
+	t = clk.get()
+
+        if not soundClip.snd is None:
+	    # first, compute how many bytes our initial chunk
+	    # to append is. ASSUMPTION: always starting from byte 0.
+	    firstbytes = min(self.bytes_per_append, len(soundClip.snd))
+	    self.total_samples = int(math.floor(len(soundClip.snd)/self.eplsound.FORMAT_SIZE))
+
+	    if self.playing:
+		# stop the playing sound 5ms prior to the new time
+		timing.timedCall(t-5, self.playStop, False)
+	    self.playing = True
+	    self.eplsound.resetSamplesPlayed()
+	    (timeInterval, appended) = timing.timedCall(t,
+						       self.eplsound.append,
+						       soundClip.snd[0:firstbytes],
+						       len(soundClip.snd[0:firstbytes])/self.eplsound.FORMAT_SIZE,
+						       0, ampFactor)
+    
+            if doDelay:
+                # accumulate the error
+                clk.accumulatedTimingError += timeInterval[0]-t
+                # tare the clock and delay the proper amount
+                clk.tare(timeInterval[0])
+                clk.delay(soundClip.getDuration())
+
+	    # it would be great if the soundClip knew the formatsize...
+	    # mark the offset into the sound clip
+            self.startInd = appended*self.eplsound.FORMAT_SIZE
+            self.endInd = len(soundClip.snd) #self.total_samples*self.eplsound.FORMAT_SIZE
+
+            # Add the callback to continue playing
+            self.last_play = timeInterval[0]
+            #addPollCallback(self.__playLoopCallback__, soundClip.snd, 0, ampFactor)
+            addPollCallback(self.__playLoopCallback__, 0, ampFactor)
+		
+            dur = soundClip.getDuration()
+
+        else:
+            dur = 0
+            
+        # log message        
+        self.logMessage("%s\t%s\t%s" % ("P",shortName,dur), timeInterval)
+
+        return timeInterval
+
+    def __playLoopCallback__(self, ow, ampFactor):
+	"""
+	Timer for appending the remainder of a sound.
+	"""
+    	currentTime = timing.now()
+
+    	if self.playing and currentTime >= self.last_play + self.play_interval:
+	    # see if stop the time
+	    if self.startInd < self.endInd:
+                # do the sound
+                s = self.currentClip.snd
+
+		# determine how much to append
+                toplay = self.eplsound.getBufferUsed()
+                toappend = self.bytes_per_append - toplay
+                if toappend <= 0:
+                    return
+                
+                actualInd = self.startInd + toappend  # self.bytes_per_append
+
+		# make sure it's not beyond the end
+		if actualInd > self.endInd:
+		    # just set to the end
+		    actualInd = self.endInd
+		
+		# append the sound
+		appended = self.eplsound.append(s[self.startInd:actualInd], len(s[self.startInd:actualInd])/self.eplsound.FORMAT_SIZE, 0, ampFactor)
+
+                self.last_play = currentTime
+		
+		# update the startInd
+		if appended > 0:
+		    self.startInd += appended*self.eplsound.FORMAT_SIZE
+		
+	    else:
+		# no more sound, so start again right away
+                self.startInd = 0
+
+
         
     def startRecording(self, basename = None, t = None, **sfargs):
         """
@@ -768,4 +940,56 @@ class AudioTrack(textlog.LogTrack):
         (r,stoptime) = self.stopRecording(t = t + duration)
         return (r,starttime)
 
+    def combineClips(self,clips):
+        """
+        Combine a list of AudioClips together, aligning to the beginning of both
+        and returning a clip the length of the longer clip.
+        """
+
+        # start with None
+        base = None
+
+        for clipinfo in clips:
+            # split out the info
+            if isinstance(clipinfo,tuple):
+                clip = clipinfo[0]
+                clipOffset = clipinfo[1]
+            else:
+                clip = clipinfo
+                clipOffset = 0
+
+            # convert from str to numpy array
+            toadd = clip.stripChannel(clip.snd)
+            toadd = numpy.array(struct.unpack(str(len(toadd)/clip.sampleWidth) + 'h', toadd), dtype=numpy.int64)
+
+            # add in on beginning if necessary
+            if clipOffset > 0:
+                toadd = numpy.concatenate([numpy.zeros(clipOffset),toadd])
+
+            if base is None:
+                base = toadd
+            else:
+                # see the diff in length
+                clipdiff = len(base) - len(toadd)
+
+                if clipdiff > 0:
+                    # append zeros to the clip
+                    toadd = numpy.concatenate([toadd,numpy.zeros(clipdiff)])
+                elif clipdiff < 0:
+                    # append zeros to the base
+                    base = numpy.concatenate([base,numpy.zeros(-clipdiff)])
+
+                # add the new signal
+                base = base + toadd
+
+        # normalize the clips to the max
+        maxval = numpy.abs(base).max()
+        if maxval > self.eplsound.SCALE:
+            base = base * self.eplsound.SCALE / maxval
+
+        # convert back to int16
+        base = base.astype(numpy.int16)
+
+        return AudioClip(clips[0].duplicateChannel(base))
+
 
